if VERSION ≥ v"0.7-"
    import LinearAlgebra.BLAS.@blasfunc
else
    import Base.BLAS.@blasfunc
end

for (fname, elty) in ((:dgbmv_,:Float64),
                      (:sgbmv_,:Float32),
                      (:zgbmv_,:ComplexF64),
                      (:cgbmv_,:ComplexF32))
    @eval begin
             # SUBROUTINE DGBMV(TRANS,M,N,KL,KU,ALPHA,A,LDA,X,INCX,BETA,Y,INCY)
             # *     .. Scalar Arguments ..
             #       DOUBLE PRECISION ALPHA,BETA
             #       INTEGER INCX,INCY,KL,KU,LDA,M,N
             #       CHARACTER TRANS
             # *     .. Array Arguments ..
             #       DOUBLE PRECISION A(LDA,*),X(*),Y(*)
        function gbmv!(trans::Char, m::Int, n::Int, kl::Int, ku::Int, alpha::($elty),
                       A::Ptr{$elty}, st::Int,
                       x::Ptr{$elty}, incx::Int, beta::($elty), y::Ptr{$elty}, incy::Int)
            ccall((@blasfunc($fname), libblas), Nothing,
                (Ref{UInt8}, Ref{BlasInt}, Ref{BlasInt}, Ref{BlasInt},
                 Ref{BlasInt}, Ref{$elty}, Ptr{$elty}, Ref{BlasInt},
                 Ptr{$elty}, Ref{BlasInt}, Ref{$elty}, Ptr{$elty},
                 Ref{BlasInt}),
                 trans, m, n, kl,
                 ku, alpha, A, st,
                 x, incx, beta, y, incy)
            y
        end
    end
end




for (fname, elty) in ((:dsbmv_,:Float64),
                      (:ssbmv_,:Float32),
                      (:zsbmv_,:ComplexF64),
                      (:csbmv_,:ComplexF32))
    @eval begin
                # SUBROUTINE DSBMV(UPLO, N, K, ALPHA, A, LDA,
                # X, INCX, BETA, Y, INCY)
                # DOUBLE PRECISION ALPHA,BETA
                # INTEGER INCX,INCY,K,LDA,N
                # CHARACTER*1 UPLO
                # DOUBLE PRECISION A(LDA,*), X(*), Y(*)

        function sbmv!(uplo::Char, n::Int, k::Int, alpha::($elty),
                       A::Ptr{$elty}, lda::Int,
                       x::Ptr{$elty}, incx::Int, beta::($elty), y::Ptr{$elty}, incy::Int)
            ccall((@blasfunc($fname), libblas), Nothing,
                (Ref{UInt8}, Ref{BlasInt}, Ref{BlasInt},
                 Ref{$elty}, Ptr{$elty}, Ref{BlasInt},
                 Ptr{$elty}, Ref{BlasInt}, Ref{$elty},
                 Ptr{$elty}, Ref{BlasInt}),
                 uplo, n, k,
                 alpha, A, lda,
                 x, incx, beta,
                 y, incy)
            y
        end
    end
end

sbmv!(uplo::Char, k::Int, alpha::T,
      A::StridedMatrix{T}, x::StridedVector{T}, beta::T, y::StridedVector{T}) where {T<:BlasFloat} =
    sbmv!(uplo,size(A,2),k,alpha,pointer(A),max(1,stride(A,2)),x,stride(x,1),beta,y,stride(y,1))


## Triangular *

for (fname, elty) in ((:dtbmv_,:Float64),
                      (:stbmv_,:Float32),
                      (:ztbmv_,:ComplexF64),
                      (:ctbmv_,:ComplexF32))
    @eval begin
        function tbmv!(uplo::Char, trans::Char, diag::Char,
                        n::Int, k::Int, A::Ptr{$elty}, lda::Int,
                        x::Ptr{$elty}, incx::Int)
            ccall((@blasfunc($fname), libblas), Nothing,
                (Ref{UInt8}, Ref{UInt8}, Ref{UInt8},
                 Ref{BlasInt}, Ref{BlasInt},
                 Ptr{$elty}, Ref{BlasInt},
                 Ptr{$elty}, Ref{BlasInt}),
                 uplo, trans, diag,
                 n, k,
                 A, lda,
                 x, incx)
            x
        end
    end
end

## Triangular \

for (fname, elty) in ((:dtbsv_,:Float64),
                      (:stbsv_,:Float32),
                      (:ztbsv_,:ComplexF64),
                      (:ctbsv_,:ComplexF32))
    @eval begin
        function tbsv!(uplo::Char, trans::Char, diag::Char,
                        n::Int, k::Int, A::Ptr{$elty}, lda::Int,
                        x::Ptr{$elty}, incx::Int)
            ccall((@blasfunc($fname), libblas), Nothing,
                (Ref{UInt8}, Ref{UInt8}, Ref{UInt8},
                 Ref{BlasInt}, Ref{BlasInt},
                 Ptr{$elty}, Ref{BlasInt},
                 Ptr{$elty}, Ref{BlasInt}),
                 uplo, trans, diag,
                 n, k,
                 A, lda,
                 x, incx)
            x
        end
    end
end
